@@ -4,7 +4,7 @@
 #include <asm/semaphore.h>
 
 #include <linux/tcfs_fs.h>
-#include <linux/kdes.h>
+#include <linux/tcfs/cipherinterface.h>
 
 struct hash_entry *hash_table[HASH_SIZE];
 struct gid_hash_entry *gid_hash_table[HASH_SIZE];
@@ -91,7 +91,7 @@ hash_lookup(int uid)
 }
 
 struct hash_entry * 
-hash_add(int uid,char *deskey, des_key_schedule *ks)
+hash_add(int uid,char *deskey, void *ks)
 {
 	int pos;
 	struct hash_entry * htmp;
@@ -157,14 +157,14 @@ hash_rem_count(int uid)
 	if (htmp1->permanent==0 && htmp1->count==0) {/* Key must be destroyed */
 		if (htmp2==NULL) {/* The element is the first */
 			hash_table[hashpos(uid)]=htmp1->next;
-			kfree(htmp1->ks);
+			tcfs_cleanup_key (htmp1->ks);
 			kfree(htmp1);
 			up(&tcfs_hash_sem);
 			return;
 		}
 
 		htmp2->next=htmp1->next;
-		kfree(htmp1->ks);
+		tcfs_cleanup_key (htmp1->ks);
 		kfree(htmp1);
 		up(&tcfs_hash_sem);
 		return;
