@@ -21,6 +21,7 @@
 
 extern int tcfsusers;
 extern int close_fp(struct file *filp);
+extern int init_tcfs_syms (void);
 
 static int tcfs_notify_change(struct inode *, struct iattr *);
 static void tcfs_put_inode(struct inode *);
@@ -394,13 +395,19 @@ static int run_tcfsiod(void *dummy)
 int
 init_tcfs_fs(void)
 {
+	int status;
+
 	kernel_thread(run_tcfsiod, NULL, 0);
 	kernel_thread(run_tcfsiod, NULL, 0);
 	kernel_thread(run_tcfsiod, NULL, 0);
 	kernel_thread(run_tcfsiod, NULL, 0);
 	init_hash(); /* Init hash tables */
 
-	return register_filesystem(&tcfs_fs_type);
+	if ((status=register_filesystem (&tcfs_fs_type))==0) {
+		status=init_tcfs_syms ();
+	}
+
+	return status;
 }
 
 void
