@@ -6,7 +6,6 @@
 #include <linux/sched.h>
 #include <linux/mm.h>
 #include <linux/malloc.h>
-#include <linux/kdes.h>
 #include <linux/tcfs_fs.h>
 
 #include <linux/locks.h>
@@ -16,27 +15,21 @@
 #define __KERNEL_SYSCALLS__
 #include <linux/unistd.h>
 /* These are the function needed to handle keys hash table */ 
-#include "hash.h"
+#include <linux/tcfs/hash.h>
+#include <linux/tcfs/cipherinterface.h>
 
-/* Implementation of TCFS ioctl functions. 
-   Currently we have to handle some ioctl to put des keys into the
-   kernel. Moreover we have to handle some ioctl for the extended 
-   attributes. Here we have GETFLAGS and SETFLAGS to handle these
-   attributes in a way compatible with the ext2 extended attributes.
-   This means that one can use TCFS as a standard NFS file system
-   and handle ext2 extended attributes via NFS as they are local.
-   When we will develop XATTRD for other filesystem we will make it
-   compatible with ext2 way so one can use ext2 extended attrs on
-   filesystems other then ext2 !!!! */
+int tcfsusers;
 
-int tcfs_ioctl(struct inode * inode, struct file * filp, unsigned int cmd,
+int
+tcfs_ioctl(struct inode * inode, struct file * filp, unsigned int cmd,
 unsigned long arg)
 {
 	struct login lg;
 	struct tgroup gl; 
 	int uid, gid;
 	int err=0;
-	des_key_schedule * ks;
+	void *ks;
+	void *gks;
 	struct gid_hash_entry * ghtmp;
 	struct hash_entry * he;
 	unsigned int mode;
@@ -82,33 +75,59 @@ unsigned long arg)
 		return 0;
 	case TCFS_IOC_LOGIN:
 		err = verify_area(VERIFY_READ, (unsigned char *) arg, sizeof(lg));
+
+#ifdef TCFS_DEBUG
+		printk ("TCFS_debug: in TCFS_IOC_LOGIN\n");
+#endif
 		if (err) {
 			return err;
 		}
 		memcpy_fromfs(&lg,(struct login *)arg,sizeof(lg));
-		/* ARGH ... here we have all we need ... UID, LEN of the deskey and DESKEY
-		   We may initialize des and put the structure in a user key cache we have to
-		   construct */
-		ks=(des_key_schedule *)kmalloc(sizeof(des_key_schedule),GFP_KERNEL);  
+
+		ks=tcfs_init_key (lg.deskey);
+
 		if (ks==NULL) {
-			printk("TCFS: Unable to get a free page\n");
+			printk("TCFS: error initializing the key\n");
 			return -ENOMEM;
 		}
-		des_set_key((des_cblock *)lg.deskey,*ks);
+
+#ifdef TCFS_DEBUG
+		{
+			int i;
+			char *pnt;
+
+			printk("TCFS_debug: key is: ");
+
+			pnt=(char *)&ks[0];
+	 		for(i=0;i<16;i++,pnt++){	
+				printk("%x:",(unsigned int)*pnt);
+			}
+			printk("\n");	
+		
+		}
+#endif
+			
 		if (hash_add(lg.uid,lg.deskey,ks)==NULL) {
 			kfree(ks);
 		}
+
+#ifdef TCFS_DEBUG
+		printk ("TCFS_debug: exiting TCFS_IOC_LOGIN\n");
+#endif
+
+		tcfsusers++;
 		return 0;
+
 	case TCFS_IOC_LOGOUT:
-		printk("in TCFS_IOC_LOGOUT    \n");
 		err = verify_area(VERIFY_READ, (unsigned int *) arg, sizeof(uid));
 		if (err) {
 			return err;
 		}
 		memcpy_fromfs(&uid,(unsigned int *)arg,sizeof(uid));
-		printk(" TCFS_IOC_LOGOUT uid %u\n",uid);
 		hash_rem_count(uid);
+		tcfsusers--;
 		return 0;
+
 	case TCFS_IOC_FULLOUT:
 		err = verify_area(VERIFY_READ, (unsigned int *) arg, sizeof(uid));
 		if (err) {
@@ -117,11 +136,13 @@ unsigned long arg)
 		memcpy_fromfs(&uid,(unsigned int *)arg,sizeof(uid));
 		hash_rem_all(uid);
 		return 0;
+
 	case TCFS_IOC_SETFIX:
 		if ((he=hash_lookup(current->uid))==NULL)
 			return -EINVAL;
 		he->permanent=1;
 		return 0;
+
 	case TCFS_IOC_GETFIX:
 		err = verify_area(VERIFY_WRITE, (unsigned char *) arg, sizeof(unsigned char));
 		if (err) {
@@ -132,6 +153,7 @@ unsigned long arg)
 			return -EINVAL;
 		put_user(he->permanent,(unsigned char *)arg);
 		return 0;
+
 	case TCFS_IOC_GETCOUNT:
 		err = verify_area(VERIFY_WRITE, (unsigned int *) arg, sizeof(unsigned int));
 		if (err) {
@@ -144,23 +166,29 @@ unsigned long arg)
 		}
 		put_user(he->count,(unsigned int *)arg);
 		return 0;
+
 	case TCFS_IOC_DELFIX:
 		he=hash_lookup(current->uid);
 		if (he==NULL)
 			return -EINVAL;
 		hash_rem_permanent(current->uid);
 		return 0;
+
 	case TCFS_IOC_GLOGIN:
 		err = verify_area(VERIFY_READ, (unsigned char *) arg, sizeof(gl));
+#ifdef TCFS_DEBUG
+		printk ("TCFS_debug: in TCFS_IOC_GLOGIN\n");
+#endif
 		if (err) {
+			printk ("TCFS: error in GLOGIN, err=%d\n", err);
 			return err;
 		}
 		memcpy_fromfs(&gl,(struct tgroup *)arg,sizeof(gl));
-#ifdef 1
+#ifdef TCFS_DEBUG
 		{
 			int i;
-			printk ("La chiave utente appena passata e': ");
-			for (i=0;i<9;i++)
+			printk ("TCFS_debug: key got via ioctl is: ");
+			for (i=0;i<(KEYSIZE+KEYSIZE/8);i++)
 				printk ("%u:", gl.deskey[i]);
 			printk ("\n");
 		}
@@ -173,22 +201,36 @@ unsigned long arg)
 		if (ghtmp->ins<gl.soglia)
 			return 0; 
                 if (ghtmp->ins>gl.soglia) {
-/*			printk("A key already exist\n");*/
+#ifdef TCFS_DEBUG
+			printk("TCFS_debug: Key already present.\n");
+#endif
 			return 0;
 		}
-		/* If we don't reach legal number then we don't interpolate keys */
-		memset(ghtmp->gidkey,'\0',20);
+#ifdef TCFS_DEBUG
+		printk ("TCFS_debug: threshold limit reached...interpolating..\n");
+#endif
+
+		memset(ghtmp->gidkey,'\0',KEYSIZE);
 		interp(ghtmp->key,ghtmp->ins,ghtmp->gidkey);
+
+/* ???
 		if (ghtmp->ks==NULL) {
-			ks=(des_key_schedule *)kmalloc(sizeof(des_key_schedule),GFP_KERNEL);  
-			if (ks==NULL) {
+			gks=(des_key_schedule *)kmalloc(2*sizeof(des_key_schedule),GFP_KERNEL);  
+			if (gks==NULL) {
 				printk("TCFS: Unable to get a free page\n");
 				return -ENOMEM;
 			}
-			ghtmp->ks=ks;
+			ghtmp->ks=gks;
 		}
-		des_set_key((des_cblock *)ghtmp->gidkey,*(ghtmp->ks));
+*/
+
+		ghtmp->ks=tcfs_init_key (ghtmp->gidkey);
+
+#ifdef TCFS_DEBUG
+		printk ("TCFS_debug: exiting TCFS_IOC_GLOGIN\n");
+#endif
 		return 0;
+
 	case TCFS_IOC_GLOGOUT:
 		err = verify_area(VERIFY_READ, (unsigned char *) arg, sizeof(gl));
 		if (err) {
@@ -202,15 +244,16 @@ unsigned long arg)
 			gid_hash_rem(gl.gid);
 			return 0;
 		}
+
+		/* If we go below the threshold, the key gets removed */
 		if (ghtmp->ins<gl.soglia) {
-			printk("Removing group key\n");
-			ks=ghtmp->ks;
+			gks=ghtmp->ks;
 			ghtmp->ks=NULL;
-			kfree(ks);
-			memset(ghtmp->gidkey,'\0',20);
-			/* If we reach minimum number, we remove key */
+			kfree(gks);
+			memset(ghtmp->gidkey,'\0',KEYSIZE);
 		}
 		return 0;
+
 	case TCFS_IOC_GFULLOUT:
 		err = verify_area(VERIFY_READ, (unsigned int *) arg, sizeof(gid));
 		if (err) {
@@ -231,6 +274,7 @@ unsigned long arg)
 			return -EINVAL;
 		he->permanent=1;
 		return 0;
+
 	case TCFS_IOC_GGETFIX:
 		err = verify_area(VERIFY_WRITE, (unsigned int *) arg, sizeof(unsigned int));
 		if (err) {
@@ -247,6 +291,7 @@ unsigned long arg)
 			return -EINVAL;
 		put_user(he->permanent,(unsigned int *)arg);
 		return 0;
+
 	case TCFS_IOC_GGETCOUNT:
 		err = verify_area(VERIFY_WRITE, (unsigned int *) arg, sizeof(unsigned int));
 		if (err) {
@@ -263,6 +308,7 @@ unsigned long arg)
 			return -EINVAL;
 		put_user(he->count,(unsigned int *)arg);
 		return 0;
+
 	case TCFS_IOC_GDELFIX:
 		err = verify_area(VERIFY_READ, (unsigned int *) arg, sizeof(unsigned int));
 		if (err) {
