@@ -1,5 +1,5 @@
 /*
- * T.C.F.S. Utils 2.0 $Revision: 2.9 $
+ * T.C.F.S. Utils 2.0 $Revision: 2.12 $
  *
  * Authors:	Giuseppe Cattaneo, <cattaneo@udsab.dia.unisa.it>
  *		Giuseppe Persiano, <giuper@udsab.dia.unisa.it>
@@ -22,16 +22,16 @@
  */
 
 /*
- *       $Source: /home/anidel/Wip/TCFS-2.2.0a/tcfslib-0.2/src/RCS/tcfs_keymaint.c,v $
+ *       $Source: /home/anidel/Wip/TCFS-2.2.0a/tcfslib-0.3.1/src/RCS/tcfs_keymaint.c,v $
  *        $State: Exp $
  *
- *     $Revision: 2.9 $
+ *     $Revision: 2.12 $
  *       $Author: anidel $
- *         $Date: 1998/12/10 10:31:30 $
+ *         $Date: 1999/01/25 16:00:43 $
  *       $Locker:  $
  */
 
-static const char *RCSid="$Id: tcfs_keymaint.c,v 2.9 1998/12/10 10:31:30 anidel Exp $";
+static const char *RCSid="$Id: tcfs_keymaint.c,v 2.12 1999/01/25 16:00:43 anidel Exp $";
 /* -+-_== */ 
 
 #include <stdio.h>
@@ -45,7 +45,7 @@ static const char *RCSid="$Id: tcfs_keymaint.c,v 2.9 1998/12/10 10:31:30 anidel
 
 #include "tcfslib.h"
 
-#include <linux/kdes.h>
+#include <kdes.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
 
@@ -71,7 +71,7 @@ tcfs_decrypt_key (char *u, char *pwd, unsigned char *t, unsigned char *tk, unsig
   char pass[_PASSWORD_LEN], *cypher;
   char *tcfskey;
   des_key_schedule ks;
-  int keysize=(flag==GROUPKEY)? KEYSIZE+1:KEYSIZE;
+  int keysize=(flag==GROUPKEY)? KEYSIZE+KEYSIZE/8:KEYSIZE;
 
   if (!tk)
 	 return 0;
@@ -111,7 +111,7 @@ tcfs_encrypt_key (char *u, char *pw, unsigned char *key, unsigned char *ek, unsi
   char pass[_PASSWORD_LEN];
   char *tcfskey;
   des_key_schedule ks;
-  int keysize=(flag==GROUPKEY)?KEYSIZE+1:KEYSIZE;
+  int keysize=(flag==GROUPKEY)?KEYSIZE+KEYSIZE/8:KEYSIZE;
 
   if (!ek)
 	 return 0;
@@ -134,7 +134,7 @@ tcfs_encrypt_key (char *u, char *pw, unsigned char *key, unsigned char *ek, unsi
       i++;
     }
 
-  tcfskey = tcfs_encode ((char*)key, KEYSIZE+8);
+  tcfskey = tcfs_encode ((char*)key, keysize+sizeof(des_cblock));
   if (!tcfskey)
 	 return 0;
 
@@ -191,6 +191,7 @@ tcfs_enable (char *u, unsigned char *tcfskey)
 
   memcpy (user.deskey, tcfskey, KEYSIZE);
   user.uid = (getpwnam (u)->pw_uid);
+
   return tcfs_callfunction (TCFS_IOC_LOGIN, (char *) &user, NULL, NULL);
 }
 
@@ -199,14 +200,14 @@ tcfs_group_enable (char *u, tcfs_gid_t gid, int soglia, int parts, unsigned char
 {
   struct tgroup group;
 
-  memcpy (group.deskey, tcfskey, KEYSIZE+1);
+  memcpy (group.deskey, tcfskey, KEYSIZE+KEYSIZE/8);
 
 #ifdef DEBUG_TCFS
 	{
 		int i;
 
 		printf ("La chiave letta in group_enable e': ");
-		for (i=0;i<=KEYSIZE;i++) {
+		for (i=0;i<=KEYSIZE+KEYSIZE/8;i++) {
 			printf ("%u:", group.deskey[i]);
 		}
 		printf ("\n");
@@ -362,7 +363,7 @@ tcfs_group_enable_default (char *u, tcfs_gid_t gid, char *p)
 	if(!tcfs_ggetpwnam(u, gid, &t))
 		 return 0;
 
-	tkey=(unsigned char*)malloc(KEYSIZE+1);
+	tkey=(unsigned char*)malloc(KEYSIZE+KEYSIZE/8);
 
 	if (!tkey)
 		 return 0;
@@ -371,20 +372,16 @@ tcfs_group_enable_default (char *u, tcfs_gid_t gid, char *p)
 #ifdef DEBUG_TCFS
 	printf ("Utente: %s\tpassword: %s\n", u, p);
 	printf ("La chiave letta e': ");
-	for (i=0;i<=KEYSIZE;i++) {
+	for (i=0;i<=KEYSIZE+KEYSIZE/8;i++) {
 		printf ("%u:", tkey[i]);
 	}
 	printf ("\n");
 #endif
 	 
 	i=tcfs_group_enable(u, t->gid, t->soglia, t->n, tkey);
-	if (!i)
-		{
-		 free(tkey);
-		 return 0;
-		}
 
-	free(tkey);
+	free (tkey);
+
 	return i;
 }
 
